@@ -11,6 +11,9 @@ from saga.utils.singleton import Singleton
 import saga.adaptors.cpi.base
 import saga.adaptors.cpi.job
 
+import radical.utils as ru
+
+
 _ADAPTOR_NAME        = 'saga.adaptor.mock'
 _ADAPTOR_SCHEMAS     = ['mock']
 _ADAPTOR_DOC         = {}
@@ -58,5 +61,5 @@ class MockJob(saga.adaptors.cpi.job.Job):
         self._cpi_base.__init__(api, adaptor)
 
 
-# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4
+
 
